#!/bin/env -S deno run -A

import { all_properties } from "./lib/all_properties.ts";
import { ConfigFile } from "./lib/config_file.ts";
import { Guess } from "./lib/guess.ts";
import { log_from } from "./lib/utils.ts";
const log = log_from(import.meta, 0);

async function main() {
	const { input_source_path, output_path } = parse_args(Deno.args);

	log(`reading input from '${input_source_path}'.`);
	const source = await Deno.readTextFile(input_source_path);

	log("scoring properties.");
	const guess_promises = [] as Promise<Guess | null>[];
	for (const property of all_properties()) guess_promises.push(property.guess_from_source(source));
	const guesses = await Promise.all(guess_promises);

	log("Filtering guesses.");
	const filtered_guesses = guesses.filter((g) => g !== null) as Guess[];
	const guessed_config = ConfigFile.from_guesses(filtered_guesses);

	const certainty = await guessed_config.calculate_certainty(source);
	log("Certainty", certainty * 100, "%");
	log(`writing config to '${output_path}'`);
	const output = guessed_config.serialize();
	await Deno.writeTextFile(output_path, output);
}

function parse_args(args: string[]) {
	const bad_usage = () => [log("Usage: cfguesser <source_file> <config_output>"), Deno.exit(1)];
	if (args.length < 2) bad_usage();
	for (const arg of args) if (["-h", "--help"].includes(arg)) bad_usage();
	const [input_source_path, output_path] = args;
	return { input_source_path, output_path };
}

if (import.meta.main) await main();
