import { Property } from "./property.ts";
import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";

import schema from "./data/schema.json" with { type: "json" };

function schema_property_parse(input: unknown) {
	return z.object({
		type: z.literal("string"),
		enum: z.array(z.string()).or(z.undefined()),
	}).or(z.object({
		type: z.literal("integer"),
	})).or(z.object({
		type: z.literal("boolean"),
	})).or(z.object({
		type: z.literal("object"),
		properties: z.record(z.string(), z.unknown()),
	})).or(z.object({
		type: z.literal("array"),
		items: z.unknown(),
	}))
		.or(z.object({
			oneOf: z.unknown().array(),
		}))
		.parse(input);
}

const blacklist = ["Language", "InsertTrailingCommas", "QualifierAlignment"];

export function all_properties() {
	const result = [] as Property[];
	const schema_properties = schema.properties as Record<string, unknown>;
	const names = Object.keys(schema.properties) as string[];
	for (const name of names) {
		if (blacklist.includes(name)) continue;
		const parsed = schema_property_parse(schema_properties[name]);
		if ("oneOf" in parsed) continue;
		if ((parsed.type === "string") && parsed.enum !== undefined) result.push(new Property(name, parsed.enum));
		if (parsed.type === "boolean") result.push(new Property(name, ["true", "false"]));
		if (parsed.type === "integer") result.push(new Property(name, ["1", "2", "4", "8", "16", "80", "120"]));
	}
	return result;
}

Deno.test("test_all_properties", () => {
	console.log(all_properties());
});
